"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConcatWithSourcemap = void 0;
const promises_1 = require("fs/promises");
const path_1 = require("path");
const source_map_1 = require("source-map");
const unix_style_path_1 = require("./utils/unix-style-path");
const separator = '\n';
class ConcatWithSourcemap {
    constructor(bundleName) {
        this._lineOffset = 0;
        this._fileContent = '';
        this._bundleName = bundleName;
        this._bundleMapName = `${bundleName}.map`;
        this._sourceMap = new source_map_1.SourceMapGenerator({ file: (0, unix_style_path_1.unixStylePath)(bundleName) });
    }
    async addDirectory(dirPath) {
        const jsFiles = (await (0, promises_1.readdir)(dirPath)).filter(x => x.endsWith('.js') && !x.endsWith(this._bundleName));
        // Order is important
        for (const jsFile of jsFiles) {
            await this.addFile((0, path_1.join)(dirPath, jsFile));
        }
    }
    async addFile(filePath) {
        const mapFileName = `${filePath}.map`;
        const fileContent = await (0, promises_1.readFile)(filePath, 'utf8');
        const sourceMapContent = (await (0, promises_1.stat)(mapFileName)
            .then(s => s.isFile())
            .catch(() => false))
            ? await (0, promises_1.readFile)(mapFileName, 'utf8')
            : undefined;
        const unixPath = (0, unix_style_path_1.unixStylePath)(filePath);
        await this.add(unixPath, fileContent, sourceMapContent);
    }
    async saveContent(filePath) {
        await (0, promises_1.writeFile)(filePath, this.content, 'utf8');
    }
    async saveMap(filePath) {
        await (0, promises_1.writeFile)(filePath, this.sourceMap, 'utf8');
    }
    async saveFiles(outputDir) {
        await this.saveContent((0, path_1.join)(outputDir, this._bundleName));
        await this.saveMap((0, path_1.join)(outputDir, this._bundleMapName));
    }
    async add(filePath, content, sourceMap) {
        if (this._fileContent.length !== 0) {
            this._fileContent += separator;
        }
        // Remove sourceMappingURL from the content
        this._fileContent += content.replace(/^\/\/# sourceMappingURL=.*$/m, '');
        const lineCount = content.split('\n').length;
        const parsedSourceMap = sourceMap ? JSON.parse(sourceMap) : undefined;
        if (parsedSourceMap) {
            await this.addWithSourceMap(parsedSourceMap);
        }
        else if (filePath) {
            this.addWithoutSourceMap(filePath, lineCount);
        }
        this._lineOffset += lineCount;
    }
    addWithoutSourceMap(filePath, lines) {
        for (let i = 1; i <= lines; i++) {
            this._sourceMap.addMapping({
                generated: {
                    line: this._lineOffset + i,
                    column: 0,
                },
                original: {
                    line: i,
                    column: 0,
                },
                source: filePath,
            });
        }
    }
    async addWithSourceMap(parsedSourceMap) {
        const upstreamSM = await new source_map_1.SourceMapConsumer(parsedSourceMap);
        upstreamSM.eachMapping(mapping => {
            this._sourceMap.addMapping({
                generated: {
                    line: this._lineOffset + mapping.generatedLine,
                    column: mapping.generatedColumn,
                },
                original: {
                    line: mapping.originalLine,
                    column: mapping.originalColumn,
                },
                source: mapping.source,
                name: mapping.name,
            });
        });
        if (upstreamSM.sourcesContent) {
            upstreamSM.sourcesContent.forEach((sourceContent, i) => {
                this._sourceMap.setSourceContent(upstreamSM.sources[i], sourceContent);
            });
        }
    }
    get content() {
        return `${this._fileContent}\n//# sourceMappingURL=${this._bundleMapName}`;
    }
    get sourceMap() {
        return this._sourceMap.toString();
    }
}
exports.ConcatWithSourcemap = ConcatWithSourcemap;
//# sourceMappingURL=concat-with-sourcemap.js.map