import { readFile, readdir, stat, writeFile } from 'fs/promises';
import { join } from 'path';
import { SourceMapGenerator, SourceMapConsumer } from 'source-map';
import { unixStylePath } from './utils/unix-style-path';
const separator = '\n';
export class ConcatWithSourcemap {
    constructor(bundleName) {
        this._lineOffset = 0;
        this._fileContent = '';
        this._bundleName = bundleName;
        this._bundleMapName = `${bundleName}.map`;
        this._sourceMap = new SourceMapGenerator({ file: unixStylePath(bundleName) });
    }
    async addDirectory(dirPath) {
        const jsFiles = (await readdir(dirPath)).filter(x => x.endsWith('.js') && !x.endsWith(this._bundleName));
        // Order is important
        for (const jsFile of jsFiles) {
            await this.addFile(join(dirPath, jsFile));
        }
    }
    async addFile(filePath) {
        const mapFileName = `${filePath}.map`;
        const fileContent = await readFile(filePath, 'utf8');
        const sourceMapContent = (await stat(mapFileName)
            .then(s => s.isFile())
            .catch(() => false))
            ? await readFile(mapFileName, 'utf8')
            : undefined;
        const unixPath = unixStylePath(filePath);
        await this.add(unixPath, fileContent, sourceMapContent);
    }
    async saveContent(filePath) {
        await writeFile(filePath, this.content, 'utf8');
    }
    async saveMap(filePath) {
        await writeFile(filePath, this.sourceMap, 'utf8');
    }
    async saveFiles(outputDir) {
        await this.saveContent(join(outputDir, this._bundleName));
        await this.saveMap(join(outputDir, this._bundleMapName));
    }
    async add(filePath, content, sourceMap) {
        if (this._fileContent.length !== 0) {
            this._fileContent += separator;
        }
        // Remove sourceMappingURL from the content
        this._fileContent += content.replace(/^\/\/# sourceMappingURL=.*$/m, '');
        const lineCount = content.split('\n').length;
        const parsedSourceMap = sourceMap ? JSON.parse(sourceMap) : undefined;
        if (parsedSourceMap) {
            await this.addWithSourceMap(parsedSourceMap);
        }
        else if (filePath) {
            this.addWithoutSourceMap(filePath, lineCount);
        }
        this._lineOffset += lineCount;
    }
    addWithoutSourceMap(filePath, lines) {
        for (let i = 1; i <= lines; i++) {
            this._sourceMap.addMapping({
                generated: {
                    line: this._lineOffset + i,
                    column: 0,
                },
                original: {
                    line: i,
                    column: 0,
                },
                source: filePath,
            });
        }
    }
    async addWithSourceMap(parsedSourceMap) {
        const upstreamSM = await new SourceMapConsumer(parsedSourceMap);
        upstreamSM.eachMapping(mapping => {
            this._sourceMap.addMapping({
                generated: {
                    line: this._lineOffset + mapping.generatedLine,
                    column: mapping.generatedColumn,
                },
                original: {
                    line: mapping.originalLine,
                    column: mapping.originalColumn,
                },
                source: mapping.source,
                name: mapping.name,
            });
        });
        if (upstreamSM.sourcesContent) {
            upstreamSM.sourcesContent.forEach((sourceContent, i) => {
                this._sourceMap.setSourceContent(upstreamSM.sources[i], sourceContent);
            });
        }
    }
    get content() {
        return `${this._fileContent}\n//# sourceMappingURL=${this._bundleMapName}`;
    }
    get sourceMap() {
        return this._sourceMap.toString();
    }
}
//# sourceMappingURL=concat-with-sourcemap.js.map